/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "webhtmltabbednavigation.h"

#include <qwebelement.h>
#include <qwebframe.h>
#include <qwebpage.h>

namespace WRT {

/*!
    \class WebHtmlTabIndexedNavigation
    \since cwrt 1.0
    \brief cwrt navigation.

    \sa WebNavigation, WebTouchNavigation, WebCursorNavigation, WebDirectionalNavigation
*/
WebHtmlTabIndexedNavigation::WebHtmlTabIndexedNavigation(QObject* object, QWebPage* webPage) :
m_viewObject(object)
, m_webPage(webPage)
, m_radioKeyPressed(Qt::Key_Down)
, m_radioGroupFocused(false)
{
    Q_ASSERT(m_viewObject != 0);
    m_viewObject->installEventFilter(this);
}

/*!
*/
WebHtmlTabIndexedNavigation::~WebHtmlTabIndexedNavigation()
{
    m_viewObject->removeEventFilter(this);
}

bool WebHtmlTabIndexedNavigation::eventFilter(QObject *object, QEvent *event)
{
    if (object == m_viewObject) {
        switch (event->type()) {
            case QEvent::KeyPress: {
                QKeyEvent* ev = static_cast<QKeyEvent*>(event);
                keyPressEvent(ev);
                return ev->isAccepted();
            }
            case QEvent::KeyRelease: {
                QKeyEvent* ev = static_cast<QKeyEvent*>(event);
                keyReleaseEvent(ev);
                return ev->isAccepted();
            }
            default:
                break;
        }
    }
    return false;
}

/*!
    If the key is directional up or left sends a tab key press with a shift modifier to QWebPage
    If the key is directional right or down sends a tab key press with out a shift modifer to QWebPage
    Otherwise sends the key press event to QWebPage
*/
void WebHtmlTabIndexedNavigation::keyPressEvent(QKeyEvent* ev)
{
    switch (ev->key()) {
    case Qt::Key_Up:
    case Qt::Key_Left: {
        QKeyEvent tabEvent(QEvent::KeyPress, Qt::Key_Tab, Qt::ShiftModifier);
        m_webPage->event(&tabEvent);
    }
    break;
    case Qt::Key_Down:
    case Qt::Key_Right: {
        QKeyEvent tabEvent(QEvent::KeyPress, Qt::Key_Tab, Qt::NoModifier);
        m_webPage->event(&tabEvent);
    }
    break;
    case Qt::Key_Return:
    case Qt::Key_Enter:
    case Qt::Key_Select: {
        QWebElement webElement = m_webPage->mainFrame()->findFirstElement(":focus");
        QString attribute = webElement.attribute("type");
        if (attribute.toLower().compare("checkbox")==0) {
            QKeyEvent spaceEvent(QEvent::KeyPress, Qt::Key_Space, Qt::NoModifier);
            m_webPage->event(&spaceEvent);
        } else if (attribute.toLower().compare("radio")==0) {
            m_radioKeyPressed = (m_radioGroupFocused) ? Qt::Key_Down : Qt::Key_Up;
            QKeyEvent keyevent(QEvent::KeyPress, m_radioKeyPressed, Qt::NoModifier);
            m_webPage->event(&keyevent);
            m_radioGroupFocused = (webElement.document().findFirst(":checked") != webElement) ? m_radioGroupFocused : !m_radioGroupFocused;
        } else if (webElement.tagName().toLower().compare("select")==0) {
            QKeyEvent keyevent(QEvent::KeyPress, Qt::Key_Down, Qt::NoModifier);
            m_webPage->event(&keyevent);
        }
    }
    break;
    default:
        if (m_webPage)
            m_webPage->event(ev);
    break;
    };
}

/*!
    If the key is directional up or left sends a tab key release with a shift modifier to QWebPage
    If the key is directional right or down sends a tab key release with out a shift modifer to QWebPage
    Otherwise sends the key press event to QWebPage
*/
void WebHtmlTabIndexedNavigation::keyReleaseEvent(QKeyEvent* ev)
{
    switch (ev->key()) {
    case Qt::Key_Up:
    case Qt::Key_Left: {
        QKeyEvent tabEvent(QEvent::KeyRelease, Qt::Key_Tab, Qt::ShiftModifier);
        m_webPage->event(&tabEvent);
    }
    break;
    case Qt::Key_Down:
    case Qt::Key_Right: {
        QKeyEvent tabEvent(QEvent::KeyRelease, Qt::Key_Tab, Qt::NoModifier);
        m_webPage->event(&tabEvent);
    }
    break;
    case Qt::Key_Return:
    case Qt::Key_Enter:
    case Qt::Key_Select: {
        QWebElement webElement = m_webPage->mainFrame()->findFirstElement(":focus");
        QString attribute = webElement.attribute("type");
        if (attribute.toLower().compare("checkbox")==0) {
            QKeyEvent spaceEvent(QEvent::KeyRelease, Qt::Key_Space, Qt::NoModifier);
            m_webPage->event(&spaceEvent);
        } else if (attribute.toLower().compare("radio")==0 || attribute.toLower().compare("select")==0) {
            QKeyEvent keyevent(QEvent::KeyRelease, m_radioKeyPressed, Qt::NoModifier);
            m_webPage->event(&keyevent);
        } else if (webElement.tagName().toLower().compare("select")==0) {
            QKeyEvent keyevent(QEvent::KeyRelease, Qt::Key_Down, Qt::NoModifier);
            m_webPage->event(&keyevent);
        }
    }
    default:
        if (m_webPage)
            m_webPage->event(ev);
    break;
    };
}

}
